@@ -51,6 +51,8 @@
 #define BL_EXTRA        2   /* Bootloader at top in separate memory area */
 #define BL_SPECIFIC     3   /* Any value greater than this is a specific start address */
 
+extern int debug;       /* defined in main.c */
+
 struct option_mapping_structure {
     const char *name;
     int32_t value;
@@ -172,6 +174,12 @@ static struct target_mapping_structure target_map[] = {
     { "atxmega384c3",   tar_atxmega384c3,   ADC_XMEGA, 0x2FDB, 0x03eb, 0x60000, 0x2000, BL_EXTRA,  512,  32, 0x1000 },
     { "atxmega16c4",    tar_atxmega16c4,    ADC_XMEGA, 0x2FD8, 0x03eb,  0x4000, 0x1000, BL_EXTRA,  256,  32,  0x400 },
     { "atxmega32c4",    tar_atxmega32c4,    ADC_XMEGA, 0x2FD9, 0x03eb,  0x8000, 0x1000, BL_EXTRA,  256,  32,  0x400 },
+    // Name             ID (arguments.h)    DevType    PID     VID     MemSize  BootSz  BootLoc  FPage EPage  ESize
+    // NOTE : that support for these targets is experimental but has been tested with stm32f4 chips on ubuntu
+    { "stm32f4_B",      tar_stm32f4_B,      DC_STM32,  0xdf11, 0x0483, 0x20000, 0x0000, BL_EXTRA,  512,   0,      0 },
+    { "stm32f4_C",      tar_stm32f4_C,      DC_STM32,  0xdf11, 0x0483, 0x40000, 0x0000, BL_EXTRA,  512,   0,      0 },
+    { "stm32f4_E",      tar_stm32f4_E,      DC_STM32,  0xdf11, 0x0483, 0x80000, 0x0000, BL_EXTRA,  512,   0,      0 },
+    { "stm32f4_G",      tar_stm32f4_G,      DC_STM32,  0xdf11, 0x0483, 0x100000,0x0000, BL_EXTRA,  512,   0,      0 },
     { NULL }
     // END_TARGET_LIST_LINE .. used for autocompletion script
 };
@@ -194,6 +202,8 @@ static struct option_mapping_structure command_map[] = {
     { "setfuse",      com_setfuse   },
     { "setsecure",    com_setsecure },
     { "start",        com_start_app },
+    { "bin2hex",      com_bin2hex   },
+    { "hex2bin",      com_hex2bin   },
     { NULL }
 };
 
@@ -270,6 +280,7 @@ static void list_targets(int mode)
             case ADC_AVR:   dev_type_name = "AVR";   break;
             case ADC_AVR32: dev_type_name = "AVR32"; break;
             case ADC_XMEGA: dev_type_name = "XMEGA"; break;
+            case DC_STM32:  dev_type_name = "STM32F4"; break;
             default:        dev_type_name = NULL;    break;
             }
             if( dev_type_name != NULL ) {
@@ -347,6 +358,8 @@ static void usage()
     fprintf( stderr, "        flash        [--force] [(flash)|--user|--eeprom]\n"
                      "                     [--suppress-validation]\n"
                      "                     [--suppress-bootloader-mem]\n"
+                     "                     [--validate-first]\n"
+                     "                     [--ignore-outside]\n"
                      "                     [--serial=hexdigits:offset] {file|STDIN}\n" );
     fprintf( stderr, "        setsecure\n" );
     fprintf( stderr, "        configure {BSB|SBV|SSB|EB|HSB}"
@@ -414,7 +427,7 @@ static int32_t assign_target( struct programmer_arguments *args,
               /* The target name includes USB bus and address info.
                * This is used to differentiate between multiple dfu
                * devices with the same vendor/chip ID numbers. By
-               * specifying the bus and address, mltiple units can
+               * specifying the bus and address, multiple units can
                * be programmed at one time.
                */
               int bus = 0;
@@ -422,7 +435,7 @@ static int32_t assign_target( struct programmer_arguments *args,
               if( 2 != sscanf(&value[name_len+1], "%i,%i", &bus, &address) )
                 return -1;
               if (bus <= 0) return -1;
-              if (address <= 0) return -1;
+              if (address < 0) return -1;
               args->bus_id = bus;
               args->device_address = address;
             }
@@ -464,6 +477,7 @@ static int32_t assign_target( struct programmer_arguments *args,
                 args->memory_address_bottom = args->flash_address_bottom;
                 args->memory_address_top = args->bootloader_top;
             }
+
             switch( args->device_type ) {
                 case ADC_8051:
                     strncpy( args->device_type_string, "8051",
@@ -489,6 +503,15 @@ static int32_t assign_target( struct programmer_arguments *args,
                     args->initial_abort = true;
                     //args->honor_interfaceclass = false;
                     break;
+                case DC_STM32:
+                    strncpy( args->device_type_string, "STM32",
+                             DEVICE_TYPE_STRING_MAX_LENGTH );
+                    break;
+                default :
+                // cSpell:words UNKNO
+                    strncpy( args->device_type_string, "UNKNO",
+                             DEVICE_TYPE_STRING_MAX_LENGTH );
+                    break;
             }
             /* There have been several reports on the mailing list of dfu-programmer
                reporting "No device present" when there clearly is. It seems Atmel's
@@ -526,7 +549,7 @@ static int32_t assign_global_options( struct programmer_arguments *args,
     for( i = 0; i < argc; i++ ) {
         if( 0 == strcmp("--suppress-bootloader-mem", argv[i]) ) {
             *argv[i] = '\0';
-            args->suppressbootloader = 1;
+            args->suppressBootloader = 1;
             break;
         }
     }
@@ -557,6 +580,46 @@ static int32_t assign_global_options( struct programmer_arguments *args,
         }
     }
 
+    /* Find '--validate-first' if it is here - even though it is not
+     * used by all this is easier. */
+    for( i = 0; i < argc; i++ ) {
+        if( 0 == strcmp("--validate-first", argv[i]) ) {
+            *argv[i] = '\0';
+
+            switch( args->command ) {
+                case com_flash:
+                case com_eflash:
+                case com_user:
+                    args->com_flash_data.validate_first = 1;
+                    break;
+                default:
+                    /* not supported. */
+                    return -1;
+            }
+            break;
+        }
+    }
+
+    /* Find '--ignore-outside' if it is here - even though it is not
+     * used by all this is easier. */
+    for( i = 0; i < argc; i++ ) {
+        if( 0 == strcmp("--ignore-outside", argv[i]) ) {
+            *argv[i] = '\0';
+
+            switch( args->command ) {
+                case com_flash:
+                case com_eflash:
+                case com_user:
+                    args->com_flash_data.ignore_outside = 1;
+                    break;
+                default:
+                    /* not supported. */
+                    return -1;
+            }
+            break;
+        }
+    }
+
     /* Find '--bin' for read binary */
     for( i = 0; i < argc; i++ ) {
         if( 0 == strcmp("--bin", argv[i]) ) {
@@ -590,6 +653,9 @@ static int32_t assign_global_options( struct programmer_arguments *args,
                 case com_user:
                     args->com_flash_data.segment = mem_user;
                     break;
+                case com_bin2hex:
+                    args->com_convert_data.segment = mem_user;
+                    break;
                 default:
                     /* not supported. */
                     return -1;
@@ -611,6 +677,9 @@ static int32_t assign_global_options( struct programmer_arguments *args,
                 case com_user:
                     args->com_flash_data.segment = mem_eeprom;
                     break;
+                case com_bin2hex:
+                    args->com_convert_data.segment = mem_eeprom;
+                    break;
                 default:
                     /* not supported. */
                     return -1;
@@ -683,7 +752,7 @@ static int32_t assign_global_options( struct programmer_arguments *args,
 
     /* Find '--serial=<hexdigit+>:<offset>' */
     for( i = 0; i < argc; i++ ) {
-      if( 0 == strncmp("--serial=", argv[i], 9) ) {
+        if( 0 == strncmp("--serial=", argv[i], 9) ) {
             *argv[i] = '\0';
 
             switch( args->command ) {
@@ -817,6 +886,18 @@ static int32_t assign_com_flash_option( struct programmer_arguments *args,
     return 0;
 }
 
+static int32_t assign_com_convert_option( struct programmer_arguments *args,
+                                          const int32_t parameter,
+                                          char *value )
+{
+    /* file */
+    args->com_convert_data.original_first_char = *value;
+    args->com_convert_data.file = value;
+
+    return 0;
+}
+
+
 static int32_t assign_com_getfuse_option( struct programmer_arguments *args,
                                       const int32_t parameter,
                                       char *value )
@@ -879,11 +960,19 @@ static int32_t assign_command_options( struct programmer_arguments *args,
                     return -3;
                 break;
 
+            case com_bin2hex:
+            case com_hex2bin:
+                required_params = 1;
+                if( 0 != assign_com_convert_option(args, param, argv[i]) )
+                    return -3;
+                break;
+
             case com_getfuse:
                 required_params = 1;
                 if( 0 != assign_com_getfuse_option(args, param, argv[i]) )
                     return -4;
                 break;
+
             case com_get:
                 required_params = 1;
                 if( 0 != assign_com_get_option(args, param, argv[i]) )
@@ -981,7 +1070,7 @@ int32_t parse_arguments( struct programmer_arguments *args,
     args->target  = tar_none;
     args->command = com_none;
     args->quiet   = 0;
-    args->suppressbootloader = 0;
+    args->suppressBootloader = 0;
 
     /* Special case - check for the help commands which do not require a device type */
     if( argc == 2 ) {
@@ -1031,6 +1120,26 @@ int32_t parse_arguments( struct programmer_arguments *args,
     *argv[1] = '\0';
     *argv[2] = '\0';
 
+    /* assign command specific default values */
+    switch( args->command ) {
+        case com_flash :
+            args->com_flash_data.force = 0;
+            args->com_flash_data.segment = mem_flash;
+            break;
+        case com_launch :
+            args->com_launch_config.noreset = 0;
+            break;
+        case com_dump :
+            args->com_read_data.segment = mem_flash;
+            args->com_flash_data.force = 0;
+            break;
+        case com_bin2hex :
+            args->com_convert_data.segment = mem_flash;
+            break;
+        default :
+            break;
+    }
+
     if( 0 != assign_global_options(args, argc, argv) ) {
         status = -5;
         goto done;
@@ -1051,13 +1160,14 @@ int32_t parse_arguments( struct programmer_arguments *args,
     }
 
     /* if this is a flash command, restore the filename */
-    if( (com_flash == args->command) || (com_eflash == args->command) || (com_user == args->command) ) {
+    if( (com_flash == args->command) || (com_eflash == args->command)
+            || (com_user == args->command) ) {
         if( 0 == args->com_flash_data.file ) {
 // TODO : it should be ok to not have a filename if --serial=hexdigits:offset is
 // provided, this should be implemented.. in fact, given that most of this
 // program is written to use a single command by it self, this probably should
 // be separated out as a new command.  The caveat is if data is written to '\0'
-// in the hex file, serialize will do nothing bc can't unwrite w/o erase
+// in the hex file, serialize will do nothing bc can't un-write w/o erase
             fprintf( stderr, "flash filename is missing\n" );
             status = -8;
             goto done;
@@ -1065,6 +1175,15 @@ int32_t parse_arguments( struct programmer_arguments *args,
         args->com_flash_data.file[0] = args->com_flash_data.original_first_char;
     }
 
+    if( (com_bin2hex == args->command) || (com_hex2bin == args->command) ) {
+        if( 0 == args->com_convert_data.file ) {
+            fprintf( stderr, "conversion filename is missing\n" );
+            status = -8;
+            goto done;
+        }
+        args->com_convert_data.file[0] = args->com_convert_data.original_first_char;
+    }
+
 done:
     if( 1 < debug ) {
         print_args( args );
