@@ -37,18 +37,16 @@
  *  launch [--no-reset]
  */
 
-// ERROR CODES :
-#define SUCCESS                             0
-#define UNSPECIFIED_ERROR                   1
-#define ARGUMENT_ERROR                      2
-#define DEVICE_ACCESS_ERROR                 3
-#define BUFFER_INIT_ERROR                   4
-#define FLASH_READ_ERROR                    5
-#define FLASH_WRITE_ERROR                   6
-#define VALIDATION_ERROR_IN_REGION          7
-#define VALIDATION_ERROR_OUTSIDE_REGION     8
-
-extern int debug;
+enum return_codes_enum {
+    SUCCESS = 0,
+    UNSPECIFIED_ERROR,                  /* general error */
+    ARGUMENT_ERROR,                     /* invalid command for target etc. */
+    DEVICE_ACCESS_ERROR,                /* security bit etc. */
+    BUFFER_INIT_ERROR,                  /* hex files problems etc. */
+    FLASH_READ_ERROR,
+    FLASH_WRITE_ERROR,
+    VALIDATION_ERROR_IN_REGION,
+    VALIDATION_ERROR_OUTSIDE_REGION };
 
 enum targets_enum { tar_at89c51snd1c,
                     tar_at89c51snd2c,
@@ -127,12 +125,16 @@ enum targets_enum { tar_at89c51snd1c,
                     tar_atxmega384c3,
                     tar_atxmega16c4,
                     tar_atxmega32c4,
+                    tar_stm32f4_B,
+                    tar_stm32f4_C,
+                    tar_stm32f4_E,
+                    tar_stm32f4_G,
                     tar_none };
 
 enum commands_enum { com_none, com_erase, com_flash, com_user, com_eflash,
                      com_configure, com_get, com_getfuse, com_dump, com_edump,
                      com_udump, com_setfuse, com_setsecure, com_start_app,
-                     com_reset, com_launch, com_read };
+                     com_reset, com_launch, com_read, com_hex2bin, com_bin2hex };
 
 enum configure_enum { conf_BSB = ATMEL_SET_CONFIG_BSB,
                       conf_SBV = ATMEL_SET_CONFIG_SBV,
@@ -176,7 +178,7 @@ struct programmer_arguments {
     /* command-specific state */
     enum commands_enum command;
     char quiet;
-    char suppressbootloader;
+    char suppressBootloader;
 
     union {
         struct com_configure_struct {
@@ -192,7 +194,7 @@ struct programmer_arguments {
 
         struct com_read_struct {
             dfu_bool bin;
-            dfu_bool force;
+            dfu_bool force;             /* do not remove blank pages */
             enum atmel_memory_unit_enum segment;
         } com_read_data;
 
@@ -216,9 +218,19 @@ struct programmer_arguments {
                                      is on last one or two words in the user
                                      page depending on the version of the
                                      bootloader - force overwrite required */
+            dfu_bool validate_first; /* Do a validate before flashing */
+            dfu_bool ignore_outside; /* Ignore validate errors outside region */
             enum atmel_memory_unit_enum segment;
         } com_flash_data;
 
+        struct com_convert_struct {
+            size_t bin_offset;          // where the bin data starts
+            char original_first_char;
+            dfu_bool force;             /* do not remove blank pages */
+            char *file;                 // for bin2hex / hex2bin conversions
+            enum atmel_memory_unit_enum segment;    // to auto-select offset
+        } com_convert_data;
+
         struct com_get_struct {
             enum get_enum name;
         } com_get_data;
