@@ -25,6 +25,7 @@
 #include <stdint.h>
 #include "dfu-bool.h"
 #include "dfu-device.h"
+#include "intel_hex.h"
 
 #define ATMEL_USER_PAGE_OFFSET 0x80800000
 
@@ -72,27 +73,6 @@ typedef struct {
     int32_t isp_force;          // Start the ISP no matter what
 } atmel_avr32_fuses_t;
 
-typedef struct {
-    size_t total_size;          // the total size of the buffer
-    size_t  page_size;          // the size of a flash page
-    uint32_t block_start;       // the start addr of a transfer
-    uint32_t block_end;         // the end addr of a transfer
-    uint32_t data_start;        // the first valid data addr
-    uint32_t data_end;          // the last valid data addr
-    uint32_t valid_start;       // the first valid memory addr
-    uint32_t valid_end;         // the last valid memory addr
-} atmel_buffer_info_t;
-
-typedef struct {
-    atmel_buffer_info_t info;
-    uint16_t *data;
-} atmel_buffer_out_t;
-
-typedef struct {
-    atmel_buffer_info_t info;
-    uint8_t *data;
-} atmel_buffer_in_t;
-
 
 enum atmel_memory_unit_enum { mem_flash, mem_eeprom, mem_security, mem_config,
     mem_boot, mem_sig, mem_user, mem_ram, mem_ext0, mem_ext1, mem_ext2,
@@ -103,33 +83,6 @@ enum atmel_memory_unit_enum { mem_flash, mem_eeprom, mem_security, mem_config,
     "ext_cs2", "ext_cs3", "ext_cs4", "ext_cs5", "ext_cs6", "ext_cs7", "ext_df"
 
 
-int32_t atmel_init_buffer_out(atmel_buffer_out_t *bout,
-        size_t total_size, size_t page_size );
-/* intialize a buffer used to send data to flash memory
- * the total size and page size must be provided.
- * the data array is filled with 0xFFFF (an invalid memory
- * value) indicating that it is unassigned. data start and
- * data end are initialized with UINT32_MAX indicating there
- * is no valid data in the buffer.  these two values are simply
- * convenience values so the start and end of data do not need
- * to be found multiple times.
- */
-
-int32_t atmel_init_buffer_in(atmel_buffer_in_t *buin,
-        size_t total_size, size_t page_size );
-/* initialize a buffer_in, used for reading the contents of program
- * memory.  total memory size must be provided.  the data array is filled
- * with 0xFF, which is unprogrammed memory.
- */
-
-int32_t atmel_validate_buffer(atmel_buffer_in_t *buin,
-                                atmel_buffer_out_t *bout, dfu_bool quiet);
-/* compare the contents of buffer_in with buffer_out to check that a target
- * memory image matches with a memory read.
- * return 0 for full validation, positive number if data bytes outside region do
- * not validate, negative number if bytes inside region that do not validate
- */
-
 int32_t atmel_read_config( dfu_device_t *device,
                            atmel_device_info_t *info );
 /*  atmel_read_config reads in all of the configuration and Manufacturer
@@ -168,7 +121,7 @@ int32_t atmel_set_config( dfu_device_t *device,
                           const uint8_t value );
 
 int32_t atmel_read_flash( dfu_device_t *device,
-                          atmel_buffer_in_t *buin,
+                          intel_buffer_in_t *buin,
                           uint8_t mem_segment,
                           const dfu_bool quiet);
 /* read the flash from buin->info.data_start to data_end and place
@@ -200,7 +153,7 @@ int32_t atmel_secure( dfu_device_t *device );
 int32_t atmel_getsecure( dfu_device_t *device );
 
 int32_t atmel_flash( dfu_device_t *device,
-                     atmel_buffer_out_t *bout,
+                     intel_buffer_out_t *bout,
                      const dfu_bool eeprom,
                      const dfu_bool force,
                      const dfu_bool hide_progress );
@@ -215,7 +168,7 @@ int32_t atmel_flash( dfu_device_t *device,
  */
 
 int32_t atmel_user( dfu_device_t *device,
-                    atmel_buffer_out_t *bout );
+                    intel_buffer_out_t *bout );
 /* Flash data to the user page.  Provide the buffer and the size of
  * flash pages for the device (this is the size of the user page).
  * Note that only the entire user page can be flashed because it is
