@@ -28,6 +28,7 @@
 #include "commands.h"
 #include "arguments.h"
 #include "intel_hex.h"
+#include "stm32.h"
 #include "atmel.h"
 #include "util.h"
 
@@ -41,9 +42,10 @@ static int security_bit_state;
 
 // ________  P R O T O T Y P E S  _______________________________
 static int32_t execute_validate( dfu_device_t *device,
-                                 atmel_buffer_out_t *bout,
+                                 intel_buffer_out_t *bout,
                                  uint8_t mem_segment,
-                                 dfu_bool quiet );
+                                 dfu_bool quiet,
+                                 dfu_bool ignore_outside);
 /* provide an out buffer to validate and whether this is from
  * flash or eeprom data sections, also wether you want it quiet
  */
@@ -72,8 +74,8 @@ static int32_t execute_erase( dfu_device_t *device,
                               struct programmer_arguments *args ) {
     int32_t result = SUCCESS;
 
-    if( !args->com_erase_data.force ) {
-        if ( 0 == atmel_blank_check( device, args->flash_address_bottom,
+    if( !(GRP_STM32 & args->device_type) && !args->com_erase_data.force ) {
+        if( 0 == atmel_blank_check( device, args->flash_address_bottom,
                                              args->flash_address_top,
                                              args->quiet ) ) {
             if ( !args->quiet ) {
@@ -86,11 +88,18 @@ static int32_t execute_erase( dfu_device_t *device,
     DEBUG( "erase 0x%X bytes.\n",
            (args->flash_address_top - args->flash_address_bottom) );
 
-    if( 0 != (result = atmel_erase_flash( device, ATMEL_ERASE_ALL, args->quiet )) ) {
+    if( GRP_STM32 & args->device_type ) {
+        result = stm32_erase_flash( device, args->quiet );
+    } else {
+        result = atmel_erase_flash( device, ATMEL_ERASE_ALL, args->quiet );
+    }
+
+    if( 0 != result ) {
         return result;
     }
 
-    if ( !args->com_erase_data.suppress_validation ) {
+    if( !(GRP_STM32 & args->device_type) &&
+            !args->com_erase_data.suppress_validation ) {
         result = atmel_blank_check( device, args->flash_address_bottom,
                                             args->flash_address_top,
                                             args->quiet );
@@ -104,7 +113,8 @@ static int32_t execute_setsecure( dfu_device_t *device,
 
     if( ADC_AVR32 != args->device_type ) {
         DEBUG( "target doesn't support security bit set.\n" );
-        fprintf( stderr, "target doesn't support security bit set.\n" );
+        fprintf( stderr,  "Operation not supported on %s\n",
+                args->device_type_string );
         return ARGUMENT_ERROR;
     }
 
@@ -113,23 +123,25 @@ static int32_t execute_setsecure( dfu_device_t *device,
     if( result < 0 ) {
         DEBUG( "Error while setting security bit. (%d)\n", result );
         fprintf( stderr, "Error setting security bit.\n" );
-        return -1;
+        return UNSPECIFIED_ERROR;
     }
 
-    return 0;
+    return SUCCESS;
 }
 
 // TODO : split this into a new command (no file is needed) - also general
 // format of this program is that only 1 command is run at a time.. caveat is
 // that if program sets a section in memory to '\0' and serialize sets it
-// otherwise, the secion will end up '\0' unless a page erase is used.. so may
+// otherwise, the section will end up '\0' unless a page erase is used.. so may
 // need to keep this part of the flash command, but specify that serialize data
 // 'wins' over data from the hex file
-static int32_t serialize_memory_image( atmel_buffer_out_t *bout,
+static int32_t serialize_memory_image( intel_buffer_out_t *bout,
                                      struct programmer_arguments *args ) {
     uint32_t target_offset = 0;
     if( args->command == com_user )
         target_offset = ATMEL_USER_PAGE_OFFSET;
+    else if( args->device_type & GRP_STM32 )
+        target_offset = STM32_FLASH_OFFSET;
 
     if ( NULL != args->com_flash_data.serial_data ) {
         int16_t *serial_data = args->com_flash_data.serial_data;
@@ -140,22 +152,23 @@ static int32_t serialize_memory_image( atmel_buffer_out_t *bout,
         for( i=0; i < length; ++i ) {
             if ( 0 != intel_process_data(bout, serial_data[i],
                         target_offset, offset + i) ) {
-                return -1;
+                return BUFFER_INIT_ERROR;
             }
         }
     }
-    return 0;
+    return SUCCESS;
 }
 
 static int32_t execute_validate( dfu_device_t *device,
-                                 atmel_buffer_out_t *bout,
+                                 intel_buffer_out_t *bout,
                                  uint8_t mem_segment,
-                                 const dfu_bool quiet ) {
+                                 const dfu_bool quiet,
+                                 const dfu_bool ignore_outside) {
     int32_t retval = UNSPECIFIED_ERROR;
     int32_t result;             // result of fcn calls
-    atmel_buffer_in_t buin;     // buffer in for storing read mem
+    intel_buffer_in_t buin;     // buffer in for storing read mem
 
-    if( 0 != atmel_init_buffer_in(&buin, bout->info.total_size,
+    if( 0 != intel_init_buffer_in(&buin, bout->info.total_size,
                                             bout->info.page_size ) ) {
         DEBUG("ERROR initializing a buffer.\n");
         retval = BUFFER_INIT_ERROR;
@@ -164,18 +177,27 @@ static int32_t execute_validate( dfu_device_t *device,
     buin.info.data_start = bout->info.valid_start;
     buin.info.data_end = bout->info.valid_end;
 
-    if( 0 !=  (result = atmel_read_flash(device, &buin,
-                                         mem_segment, quiet)) ) {
+    if( device->type & GRP_STM32 ) {
+        result = stm32_read_flash( device, &buin, mem_segment, quiet );
+    } else {
+        result = atmel_read_flash( device, &buin, mem_segment, quiet );
+    }
+
+    if( 0 != result ) {
         DEBUG("ERROR: could not read memory, err %d.\n", result);
         retval = FLASH_READ_ERROR;
         goto error;
     }
 
-    if( 0 != (result = atmel_validate_buffer( &buin, bout, quiet )) ) {
+    if( 0 != (result = intel_validate_buffer( &buin, bout, quiet )) ) {
         if( result < 0 ) {
             retval = VALIDATION_ERROR_IN_REGION;
         } else {
-            retval = VALIDATION_ERROR_OUTSIDE_REGION;
+            if (ignore_outside) {
+                retval = SUCCESS;
+            } else {
+                retval = VALIDATION_ERROR_OUTSIDE_REGION;
+            }
         }
         goto error;
     }
@@ -193,7 +215,7 @@ error:
     return retval;
 }
 
-static void print_flash_usage( atmel_buffer_info_t *info ) {
+static void print_flash_usage( intel_buffer_info_t *info ) {
     fprintf( stderr,
             "0x%X bytes written into 0x%X bytes memory (%.02f%%).\n",
             info->data_end - info->data_start + 1,
@@ -202,27 +224,159 @@ static void print_flash_usage( atmel_buffer_info_t *info ) {
                 (float) (info->valid_end - info->valid_start + 1)) ) ;
 }
 
+static int32_t execute_hex2bin( dfu_device_t *device,
+        struct programmer_arguments *args ) {
+    int32_t  retval = -1;
+    uint32_t  i;
+    intel_buffer_out_t bout;
+    size_t   memory_size;
+    size_t   page_size;
+    uint32_t target_offset = 0;
+
+    memory_size = args->memory_address_top + 1;
+    page_size = args->flash_page_size;
+
+    // ----------------- CONVERT HEX FILE TO BINARY -------------------------
+    if( 0 != intel_init_buffer_out(&bout, memory_size, page_size) ) {
+        DEBUG("ERROR initializing a buffer.\n");
+        goto error;
+    }
+
+    if( 0!= intel_hex_to_buffer( args->com_convert_data.file, &bout,
+                target_offset, args->quiet ) ) {
+        DEBUG( "Something went wrong with creating the memory image.\n" );
+        goto error;
+    }
+
+    if( !args->quiet )
+        fprintf( stderr, "Dumping 0x%X bytes from address offset 0x%X.\n",
+                bout.info.data_end + 1, target_offset );
+    for( i = 0; i <= bout.info.data_end; i++ ) {
+        fprintf( stdout, "%c", bout.data[i] <= 0xFF ? bout.data[i] & 0xFF : 0xFF);
+    }
+
+    fflush( stdout );
+
+    retval = 0;
+
+error:
+    if( NULL != bout.data ) {
+        free( bout.data );
+        bout.data = NULL;
+    }
+
+    return retval;
+}
+
+static int32_t execute_bin2hex( dfu_device_t *device,
+        struct programmer_arguments *args ) {
+    int32_t retval = -1;        // return value for this fcn
+    intel_buffer_in_t buin;     // buffer in for storing read mem
+    enum atmel_memory_unit_enum mem_segment = args->com_convert_data.segment;
+    size_t mem_size = 0;
+    size_t page_size = 0;
+    uint32_t target_offset = 0; // address offset on the target device
+        // NOTE: target_offset may not be set appropriately for device
+        // classes other than ADC_AVR32
+    FILE *fp = NULL;
+    char *filename = args->com_convert_data.file;
+
+    if( ADC_AVR32 == args->device_type ) {
+        target_offset = 0x80000000;
+    }
+
+    switch( mem_segment ) {
+        case mem_flash:
+            mem_size = args->memory_address_top + 1;
+            page_size = args->flash_page_size;
+            break;
+        case mem_eeprom:
+            mem_size = args->eeprom_memory_size;
+            page_size = args->eeprom_page_size;
+            break;
+        case mem_user:
+            mem_size = args->flash_page_size;
+            page_size = args->flash_page_size;
+            target_offset = 0x80800000;
+            break;
+        default:
+            fprintf( stderr, "Dump not currently supported for this memory.\n" );
+            goto error;
+    }
+
+    if( 0 != intel_init_buffer_in(&buin, mem_size, page_size) ) {
+        DEBUG("ERROR initializing a buffer.\n");
+        goto error;
+    }
+
+    if( mem_segment == mem_flash ) {
+        buin.info.data_start = args->flash_address_bottom;
+        buin.info.data_end = args->flash_address_top;
+    }
+
+    if( NULL == filename ) {
+        if( !args->quiet ) fprintf( stderr, "Invalid filename.\n" );
+        retval = -2;
+        goto error;
+    }
+
+    if( 0 == strcmp("STDIN", filename) ) {
+        fp = stdin;
+    } else {
+        fp = fopen( filename, "r" );
+        if( NULL == fp ) {
+            if( !args->quiet ) fprintf( stderr, "Error opening %s\n", filename );
+            retval = -3;
+            goto error;
+        }
+    }
+
+    buin.info.data_end = fread(buin.data, 1, buin.info.total_size, fp);
+    if( buin.info.data_end == 0 ) {
+        if( !args->quiet ) fprintf( stderr, "ERROR: no bytes read\n" );
+        retval = -4;
+        goto error;
+    }
+
+    if( !args->quiet )
+        fprintf( stderr, "Read 0x%X bytes, making hex with address offset 0x%X.\n",
+                buin.info.data_end + 1, target_offset );
+
+    retval = intel_hex_from_buffer( &buin, args->com_convert_data.force, target_offset );
+
+error:
+    if( NULL != buin.data ) {
+        free( buin.data );
+        buin.data = NULL;
+    }
+
+    return retval;
+}
+
 static int32_t execute_flash( dfu_device_t *device,
                                 struct programmer_arguments *args ) {
     int32_t  retval = UNSPECIFIED_ERROR;
     int32_t  result;
     uint32_t  i;
-    atmel_buffer_out_t bout;
+    intel_buffer_out_t bout;
     size_t   memory_size;
     size_t   page_size;
     enum atmel_memory_unit_enum mem_type = args->com_flash_data.segment;
     uint32_t target_offset = 0;
 
-    // assign the correct memory size
+    /* assign the correct memory size */
     switch ( mem_type ) {
         case mem_flash:
+            if( args->device_type & GRP_STM32 ) {
+                target_offset = STM32_FLASH_OFFSET;
+            }
             memory_size = args->memory_address_top + 1;
             page_size = args->flash_page_size;
             break;
         case mem_eeprom:
             if( 0 == args->eeprom_memory_size ) {
                 fprintf( stderr, "This device has no eeprom.\n" );
-                return -1;
+                return ARGUMENT_ERROR;
             }
             memory_size = args->eeprom_memory_size;
             page_size = args->eeprom_page_size;
@@ -239,12 +393,12 @@ static int32_t execute_flash( dfu_device_t *device,
             }
             break;
         default:
-            memory_size = 0;
-            page_size = 0;
+            DEBUG("Unknown memory type %d\n", mem_type);
+            return ARGUMENT_ERROR;
     }
 
     // ----------------- CONVERT HEX FILE TO BINARY -------------------------
-    if( 0 != atmel_init_buffer_out(&bout, memory_size, page_size) ) {
+    if( 0 != intel_init_buffer_out(&bout, memory_size, page_size) ) {
         DEBUG("ERROR initializing a buffer.\n");
         retval = BUFFER_INIT_ERROR;
         goto error;
@@ -287,7 +441,7 @@ static int32_t execute_flash( dfu_device_t *device,
         // check that there isn't anything overlapping the bootloader
         for( i = args->bootloader_bottom; i <= args->bootloader_top; i++) {
             if( bout.data[i] <= UINT8_MAX ) {
-                if( true == args->suppressbootloader ) {
+                if( true == args->suppressBootloader ) {
                     //If we're ignoring the bootloader, don't write to it
                     bout.data[i] = UINT16_MAX;
                 } else {
@@ -328,7 +482,7 @@ static int32_t execute_flash( dfu_device_t *device,
                     " Use dump-user to obtain valid configuration words.\n");
             retval = ARGUMENT_ERROR;
             goto error;
-            // TODO : implement so this error only appers when data overlaps the
+            // TODO : implement so this error only appears when data overlaps the
             // bootloader configuration words.  This would require reading the user
             // page to add that data to the buffer, and also should include
             // checking the bootloader version to make sure the right number of
@@ -348,13 +502,28 @@ static int32_t execute_flash( dfu_device_t *device,
         }
     }
 
+    // ------------------ INITIAL VALIDATE (if required) -------------------
+    if ( 1 == args->com_flash_data.validate_first ) {
+        if( 0 == ( retval = execute_validate(device, &bout, mem_type, args->quiet,
+                                             args->com_flash_data.ignore_outside)) ) {
+            goto success;
+        }
+        // Else, continue to flash
+    }
+
     // ------------------ WRITE PROGRAM DATA -------------------------------
-    if ( mem_type == mem_user ) {
+    if( mem_type == mem_user ) {
         result = atmel_user( device, &bout );
     } else {
-        result = atmel_flash(device, &bout,
-                mem_type == mem_eeprom ? true : false,
-                args->com_flash_data.force, args->quiet);
+        if( args->device_type & GRP_STM32 ) {
+            result = stm32_write_flash( device, &bout,
+                    mem_type == mem_eeprom ? true : false,
+                    args->com_flash_data.force, args->quiet );
+        } else {
+            result = atmel_flash(device, &bout,
+                    mem_type == mem_eeprom ? true : false,
+                    args->com_flash_data.force, args->quiet);
+        }
     }
     if( 0 != result ) {
         DEBUG( "Error writing %s data. (err %d)\n", "memory", result );
@@ -364,7 +533,8 @@ static int32_t execute_flash( dfu_device_t *device,
 
     // ------------------  VALIDATE PROGRAM ------------------------------
     if( 0 == args->com_flash_data.suppress_validation ) {
-        if( 0 != ( retval = execute_validate(device, &bout, mem_type, args->quiet)) ) {
+        if( 0 != ( retval = execute_validate(device, &bout, mem_type, args->quiet,
+                                             args->com_flash_data.ignore_outside)) ) {
             fprintf( stderr, "Memory did not validate. Did you erase?\n" );
             goto error;
         } else if ( 0 == args->quiet ) {
@@ -374,7 +544,7 @@ static int32_t execute_flash( dfu_device_t *device,
         print_flash_usage( &bout.info );
     }
 
-
+success:
     retval = SUCCESS;
 
 error:
@@ -397,13 +567,19 @@ static int32_t execute_getfuse( dfu_device_t *device,
     if( !(ADC_AVR32 & args->device_type) ) {
         DEBUG( "target doesn't support fuse set operation.\n" );
         fprintf( stderr, "target doesn't support fuse set operation.\n" );
-        return -1;
+        return ARGUMENT_ERROR;
     }
 
     /* Check AVR32 security bit in order to provide a better error message. */
     security_check( device );
 
-    status = atmel_read_fuses( device, &info );
+    if( args->device_type & GRP_STM32 ) {
+        fprintf( stderr, "Operation not supported on %s.\n",
+                args->device_type_string );
+        return ARGUMENT_ERROR;
+    } else {
+        status = atmel_read_fuses( device, &info );
+    }
 
     if( 0 != status ) {
         DEBUG( "Error reading %s config information.\n",
@@ -470,7 +646,13 @@ static int32_t execute_get( dfu_device_t *device,
     /* Check AVR32 security bit in order to provide a better error message. */
     security_check( device );
 
-    status = atmel_read_config( device, &info );
+    if( args->device_type & GRP_STM32 ) {
+        fprintf( stderr, "Operation not supported on %s.\n",
+                args->device_type_string );
+        return -1;
+    } else {
+        status = atmel_read_config( device, &info );
+    }
 
     if( 0 != status ) {
         DEBUG( "Error reading %s config information.\n",
@@ -571,7 +753,7 @@ static int32_t execute_dump( dfu_device_t *device,
     int32_t i = 0;
     int32_t retval = UNSPECIFIED_ERROR;
     int32_t result;             // result of fcn calls
-    atmel_buffer_in_t buin;     // buffer in for storing read mem
+    intel_buffer_in_t buin;     // buffer in for storing read mem
     enum atmel_memory_unit_enum mem_segment = args->com_read_data.segment;
     size_t mem_size = 0;
     size_t page_size = 0;
@@ -583,7 +765,11 @@ static int32_t execute_dump( dfu_device_t *device,
         case mem_flash:
             mem_size = args->memory_address_top + 1;
             page_size = args->flash_page_size;
-            if( ADC_AVR32 == args->device_type ) target_offset = 0x80000000;
+            if( ADC_AVR32 == args->device_type ) {
+                target_offset = 0x80000000;
+            } else if( GRP_STM32 & args->device_type ) {
+                target_offset = STM32_FLASH_OFFSET;
+            }
             break;
         case mem_eeprom:
             mem_size = args->eeprom_memory_size;
@@ -595,12 +781,12 @@ static int32_t execute_dump( dfu_device_t *device,
             target_offset = 0x80800000;
             break;
         default:
-            fprintf( stderr, "Dump not currenlty supported for this memory.\n" );
+            fprintf( stderr, "Dump not currently supported for this memory.\n" );
             retval = ARGUMENT_ERROR;
             goto error;
     }
 
-    if( 0 != atmel_init_buffer_in(&buin, mem_size, page_size) ) {
+    if( 0 != intel_init_buffer_in(&buin, mem_size, page_size) ) {
         DEBUG("ERROR initializing a buffer.\n");
         retval = BUFFER_INIT_ERROR;
         goto error;
@@ -611,11 +797,15 @@ static int32_t execute_dump( dfu_device_t *device,
         buin.info.data_end = args->flash_address_top;
     }
 
-    /* Check AVR32 security bit in order to provide a better error message. */
-    security_check( device );   // avr32 has no eeprom, but OK
+    if( args->device_type & GRP_STM32 ) {
+        result = stm32_read_flash(device, &buin, mem_segment, args->quiet);
+    } else {
+        /* Check AVR32 security bit in order to provide a better error message */
+        security_check( device );   // avr32 has no eeprom, but OK
+        result = atmel_read_flash(device, &buin, mem_segment, args->quiet);
+    }
 
-    if( 0 != (result = atmel_read_flash(device, &buin,
-                    mem_segment, args->quiet)) ) {
+    if( 0 != result ) {
         DEBUG("ERROR: could not read memory, err %d.\n", result);
         security_message();
         retval = FLASH_READ_ERROR;
@@ -631,7 +821,7 @@ static int32_t execute_dump( dfu_device_t *device,
             if( buin.data[i] != 0xFF ) break;
             if( i / buin.info.page_size >
                     buin.info.data_start / buin.info.page_size ) {
-                // i has just jumpped to a different page than buin.data_start
+                // i has just jumped to a different page than buin.data_start
                 buin.info.data_start = i;
             }
         }
@@ -688,17 +878,17 @@ static int32_t execute_setfuse( dfu_device_t *device,
     int32_t name = args->com_setfuse_data.name;
 
     /* only ADC_AVR32 seems to support fuse operation */
-    if( !(ADC_AVR32 & args->device_type) ) {
+    if( !(ADC_AVR32 & args->device_type) || (GRP_STM32 & args->device_type) ) {
+        fprintf( stderr,  "Operation not supported on %s\n",
+                args->device_type_string );
         DEBUG( "target doesn't support fuse set operation.\n" );
-        fprintf( stderr, "target doesn't support fuse set operation.\n" );
         return -1;
     }
 
     /* Check AVR32 security bit in order to provide a better error message. */
     security_check( device );
 
-    if( 0 != atmel_set_fuse(device, name, value) )
-    {
+    if( 0 != atmel_set_fuse(device, name, value) ) {
         DEBUG( "Fuse set failed.\n" );
         fprintf( stderr, "Fuse set failed.\n" );
         security_message();
@@ -714,8 +904,9 @@ static int32_t execute_configure( dfu_device_t *device,
     int32_t name = args->com_configure_data.name;
 
     if( ADC_8051 != args->device_type ) {
+        fprintf( stderr, "Operation not supported on %s\n",
+                args->device_type_string );
         DEBUG( "target doesn't support configure operation.\n" );
-        fprintf( stderr, "target doesn't support configure operation.\n" );
         return -1;
     }
 
@@ -737,7 +928,9 @@ static int32_t execute_configure( dfu_device_t *device,
 
 static int32_t execute_launch( dfu_device_t *device,
                                   struct programmer_arguments *args ) {
-    if ( args->com_launch_config.noreset ) {
+    if( args->device_type & GRP_STM32 ) {
+        return stm32_start_app( device, args->quiet );
+    } else if( args->com_launch_config.noreset ) {
         return atmel_start_app_noreset( device );
     } else {
         return atmel_start_app_reset( device );
@@ -750,6 +943,10 @@ int32_t execute_command( dfu_device_t *device,
     switch( args->command ) {
         case com_erase:
             return execute_erase( device, args );
+        case com_bin2hex:
+            return execute_bin2hex( device, args );
+        case com_hex2bin:
+            return execute_hex2bin( device, args );
         case com_flash:
             return execute_flash( device, args );
         case com_eflash:
