@@ -18,24 +18,20 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#if HAVE_CONFIG_H
-# include <config.h>
-#endif
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <stddef.h>
-#ifdef HAVE_LIBUSB_1_0
-#include <libusb.h>
-#else
-#include <usb.h>
-#endif
+#include <libusb-1.0/libusb.h>
 #include <errno.h>
+
 #include "dfu.h"
 #include "util.h"
 #include "dfu-bool.h"
 
+// cSpell:words DNBUSY
+
 /* DFU commands */
 #define DFU_DETACH      0
 #define DFU_DNLOAD      1
@@ -71,14 +67,9 @@
 static uint16_t transaction = 0;
 
 // ________  P R O T O T Y P E S  _______________________________
-#ifdef HAVE_LIBUSB_1_0
 static int32_t dfu_find_interface( struct libusb_device *device,
                                    const dfu_bool honor_interfaceclass,
                                    const uint8_t bNumConfigurations);
-#else
-static int32_t dfu_find_interface( const struct usb_device *device,
-                                   const dfu_bool honor_interfaceclass );
-#endif
 /*  Used to find the dfu interface for a device if there is one.
  *
  *  device - the device to search
@@ -91,7 +82,7 @@ static int32_t dfu_find_interface( const struct usb_device *device,
 static int32_t dfu_make_idle( dfu_device_t *device, const dfu_bool initial_abort );
 /*  Gets the device into the dfuIDLE state if possible.
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *
  *  returns 0 on success, 1 if device was reset, error otherwise
  */
@@ -109,7 +100,7 @@ static int32_t dfu_transfer_in( dfu_device_t *device,
                                 const size_t length );
 
 static void dfu_msg_response_output( const char *function, const int32_t result );
-/*  Used to output the response from our USB request in a human reable
+/*  Used to output the response from our USB request in a human readable
  *  form.
  *
  *  function - the calling function to output on behalf of
@@ -117,6 +108,17 @@ static void dfu_msg_response_output( const char *function, const int32_t result
  */
 
 // ________  F U N C T I O N S  _______________________________
+void dfu_set_transaction_num( uint16_t newNum ) {
+    TRACE( "%s( %u )\n", __FUNCTION__, newNum );
+    transaction = newNum;
+    DEBUG("wValue set to %d\n", transaction);
+}
+
+uint16_t dfu_get_transaction_num( void ) {
+    TRACE( "%s( %u )\n", __FUNCTION__ );
+    return transaction;
+}
+
 int32_t dfu_detach( dfu_device_t *device, const int32_t timeout ) {
     int32_t result;
 
@@ -169,9 +171,6 @@ int32_t dfu_download( dfu_device_t *device,
     dfu_msg_response_output( __FUNCTION__, result );
 
     return result;
-// TODO : consider adding dfu_get_status here and returning status instead of
-// the length of the download? if there is a length mismatch on the download it
-// can be returned as -1.. this should simplify all calls to dfu_ from atmel_
 }
 
 int32_t dfu_upload( dfu_device_t *device, const size_t length, uint8_t* data ) {
@@ -304,7 +303,6 @@ int32_t dfu_abort( dfu_device_t *device ) {
     return result;
 }
 
-#ifdef HAVE_LIBUSB_1_0
 struct libusb_device *dfu_device_init( const uint32_t vendor,
                                        const uint32_t product,
                                        const uint32_t bus_number,
@@ -313,8 +311,8 @@ struct libusb_device *dfu_device_init( const uint32_t vendor,
                                        const dfu_bool initial_abort,
                                        const dfu_bool honor_interfaceclass ) {
     libusb_device **list;
-    size_t i,devicecount;
-    extern libusb_context *usbcontext;
+    size_t i,deviceCount;
+    extern libusb_context *usbContext;
     int32_t retries = 4;
 
     TRACE( "%s( %u, %u, %p, %s, %s )\n", __FUNCTION__, vendor, product,
@@ -324,9 +322,9 @@ struct libusb_device *dfu_device_init( const uint32_t vendor,
     DEBUG( "%s(%08x, %08x)\n",__FUNCTION__, vendor, product );
 
 retry:
-    devicecount = libusb_get_device_list( usbcontext, &list );
+    deviceCount = libusb_get_device_list( usbContext, &list );
 
-    for( i = 0; i < devicecount; i++ ) {
+    for( i = 0; i < deviceCount; i++ ) {
         libusb_device *device = list[i];
         struct libusb_device_descriptor descriptor;
 
@@ -397,88 +395,6 @@ retry:
 
     return NULL;
 }
-#else
-struct usb_device *dfu_device_init( const uint32_t vendor,
-                                    const uint32_t product,
-                                    const uint32_t bus_number,
-                                    const uint32_t device_address,
-                                    dfu_device_t *dfu_device,
-                                    const dfu_bool initial_abort,
-                                    const dfu_bool honor_interfaceclass ) {
-    struct usb_bus *usb_bus;
-    struct usb_device *device;
-    int32_t retries = 4;
-
-    TRACE( "%s( %u, %u, %p, %s, %s )\n", __FUNCTION__, vendor, product,
-           dfu_device, ((true == initial_abort) ? "true" : "false"),
-           ((true == honor_interfaceclass) ? "true" : "false") );
-
-retry:
-    if( 0 < retries ) {
-        usb_find_busses();
-        usb_find_devices();
-
-        /* Walk the tree and find our device. */
-        for( usb_bus = usb_get_busses(); NULL != usb_bus; usb_bus = usb_bus->next ) {
-            for( device = usb_bus->devices; NULL != device; device = device->next) {
-                if(    (vendor  == device->descriptor.idVendor)
-                    && (product == device->descriptor.idProduct)
-                    && ((bus_number == 0)
-                        || (device->devnum == device_address
-                            && (usb_bus->location >> 24) == bus_number)))
-                {
-                    int32_t tmp;
-                    DEBUG( "found device at USB:%d,%d\n", device->devnum, (usb_bus->location >> 24) );
-                    /* We found a device that looks like it matches...
-                     * let's try to find the DFU interface, open the device
-                     * and claim it. */
-                    tmp = dfu_find_interface( device, honor_interfaceclass );
-                    if( 0 <= tmp ) {
-                        /* The interface is valid. */
-                        dfu_device->interface = tmp;
-                        dfu_device->handle = usb_open( device );
-                        if( NULL != dfu_device->handle ) {
-                            if( 0 == usb_set_configuration(dfu_device->handle, 1) ) {
-                                if( 0 == usb_claim_interface(dfu_device->handle, dfu_device->interface) ) {
-                                    switch( dfu_make_idle(dfu_device, initial_abort) )
-                                    {
-                                        case 0:
-                                            return device;
-                                        case 1:
-                                            retries--;
-                                            goto retry;
-                                    }
-
-                                    DEBUG( "Failed to put the device in dfuIDLE mode.\n" );
-                                    usb_release_interface( dfu_device->handle, dfu_device->interface );
-                                    usb_close( dfu_device->handle );
-                                    retries = 4;
-                                } else {
-                                    DEBUG( "Failed to claim the DFU interface.\n" );
-                                    usb_close( dfu_device->handle );
-                                }
-                            } else {
-                                DEBUG( "Failed to set configuration.\n");
-
-                                usb_close( dfu_device->handle );
-                            }
-                        } else {
-                            DEBUG( "Failed to open device.\n" );
-                        }
-                    } else {
-                        DEBUG( "Failed to find the DFU interface.\n" );
-                    }
-                }
-            }
-        }
-    }
-
-    dfu_device->handle = NULL;
-    dfu_device->interface = 0;
-
-    return NULL;
-}
-#endif
 
 char* dfu_state_to_string( const int32_t state ) {
     char *message = "unknown state";
@@ -578,7 +494,6 @@ char* dfu_status_to_string( const int32_t status ) {
     return message;
 }
 
-#ifdef HAVE_LIBUSB_1_0
 static int32_t dfu_find_interface( struct libusb_device *device,
                                    const dfu_bool honor_interfaceclass,
                                    const uint8_t bNumConfigurations) {
@@ -594,7 +509,7 @@ static int32_t dfu_find_interface( struct libusb_device *device,
             DEBUG( "can't get_config_descriptor: %d\n", c );
             return -1;
         }
-        DEBUG( "config %d: maxpower=%d*2 mA\n", c, config->MaxPower );
+        DEBUG( "config %d: MaxPower=%d*2 mA\n", c, config->MaxPower );
 
         /* Loop through all of the interfaces */
         for( i = 0; i < config->bNumInterfaces; i++ ) {
@@ -634,41 +549,7 @@ static int32_t dfu_find_interface( struct libusb_device *device,
 
     return -1;
 }
-#else
-static int32_t dfu_find_interface( const struct usb_device *device,
-                                   const dfu_bool honor_interfaceclass ) {
-    int32_t c, i;
-    struct usb_config_descriptor *config;
-    struct usb_interface_descriptor *interface;
 
-    /* Loop through all of the configurations */
-    for( c = 0; c < device->descriptor.bNumConfigurations; c++ ) {
-        config = &(device->config[c]);
-
-        /* Loop through all of the interfaces */
-        for( i = 0; i < config->interface->num_altsetting; i++) {
-            interface = &(config->interface->altsetting[i]);
-
-            if( true == honor_interfaceclass ) {
-                /* Check if the interface is a DFU interface */
-                if(    (USB_CLASS_APP_SPECIFIC == interface->bInterfaceClass)
-                    && (DFU_SUBCLASS == interface->bInterfaceSubClass) )
-                {
-                    DEBUG( "Found DFU Inteface: %d\n", interface->bInterfaceNumber );
-                    return interface->bInterfaceNumber;
-                }
-            } else {
-                /* If there is a bug in the DFU firmware, return the first
-                 * found interface. */
-                DEBUG( "Found DFU Inteface: %d\n", interface->bInterfaceNumber );
-                return interface->bInterfaceNumber;
-            }
-        }
-    }
-
-    return -1;
-}
-#endif
 
 static int32_t dfu_make_idle( dfu_device_t *device,
                               const dfu_bool initial_abort ) {
@@ -717,11 +598,7 @@ static int32_t dfu_make_idle( dfu_device_t *device,
             case STATE_APP_DETACH:
             case STATE_DFU_MANIFEST_WAIT_RESET:
                 DEBUG( "Resetting the device\n" );
-#ifdef HAVE_LIBUSB_1_0
                 libusb_reset_device( device->handle );
-#else
-                usb_reset( device->handle );
-#endif
                 return 1;
         }
 
@@ -737,7 +614,6 @@ static int32_t dfu_transfer_out( dfu_device_t *device,
                                  const int32_t value,
                                  uint8_t* data,
                                  const size_t length ) {
-#ifdef HAVE_LIBUSB_1_0
     return libusb_control_transfer( device->handle,
                 /* bmRequestType */ LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE,
                 /* bRequest      */ request,
@@ -746,16 +622,6 @@ static int32_t dfu_transfer_out( dfu_device_t *device,
                 /* Data          */ data,
                 /* wLength       */ length,
                                     DFU_TIMEOUT );
-#else
-    return usb_control_msg( device->handle,
-                /* bmRequestType */ USB_ENDPOINT_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
-                /* bRequest      */ request,
-                /* wValue        */ value,
-                /* wIndex        */ device->interface,
-                /* Data          */ (char*) data,
-                /* wLength       */ length,
-                                    DFU_TIMEOUT );
-#endif
 }
 
 static int32_t dfu_transfer_in( dfu_device_t *device,
@@ -763,7 +629,6 @@ static int32_t dfu_transfer_in( dfu_device_t *device,
                                 const int32_t value,
                                 uint8_t* data,
                                 const size_t length ) {
-#ifdef HAVE_LIBUSB_1_0
     return libusb_control_transfer( device->handle,
                 /* bmRequestType */ LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE,
                 /* bRequest      */ request,
@@ -772,16 +637,6 @@ static int32_t dfu_transfer_in( dfu_device_t *device,
                 /* Data          */ data,
                 /* wLength       */ length,
                                     DFU_TIMEOUT );
-#else
-    return usb_control_msg( device->handle,
-                /* bmRequestType */ USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
-                /* bRequest      */ request,
-                /* wValue        */ value,
-                /* wIndex        */ device->interface,
-                /* Data          */ (char*) data,
-                /* wLength       */ length,
-                                    DFU_TIMEOUT );
-#endif
 }
 
 static void dfu_msg_response_output( const char *function,
@@ -792,56 +647,51 @@ static void dfu_msg_response_output( const char *function,
         msg = "No error.";
     } else {
         switch( result ) {
-            case -ENOENT:
-                msg = "-ENOENT: URB was canceled by unlink_urb";
+            case LIBUSB_ERROR_IO :
+                msg = "LIBUSB_ERROR_IO: Input/output error.";
                 break;
-#ifdef EINPROGRESS
-            case -EINPROGRESS:
-                msg = "-EINPROGRESS: URB still pending, no results yet "
-                      "(actually no error until now)";
+            case LIBUSB_ERROR_INVALID_PARAM :
+                msg = "LIBUSB_ERROR_INVALID_PARAM: Invalid parameter.";
                 break;
-#endif
-#ifdef EPROTO
-            case -EPROTO:
-                msg = "-EPROTO: a) Bitstuff error or b) Unknown USB error";
+            case LIBUSB_ERROR_ACCESS :
+                msg = "LIBUSB_ERROR_ACCESS: Access denied (insufficient permissions)";
                 break;
-#endif
-            case -EILSEQ:
-                msg = "-EILSEQ: CRC mismatch";
+            case LIBUSB_ERROR_NO_DEVICE :
+                msg = "LIBUSB_ERROR_NO_DEVICE: No such device (it may have been disconnected)";
                 break;
-            case -EPIPE:
-                // think that in libusb, -9 is LIBUSB_ERROR_PIPE
-                msg = "-EPIPE: a) Babble detect or b) Endpoint stalled";
+            case LIBUSB_ERROR_NOT_FOUND :
+                msg = "LIBUSB_ERROR_NOT_FOUND: Entity not found.";
                 break;
-#ifdef ETIMEDOUT
-            case -ETIMEDOUT:
-                msg = "-ETIMEDOUT: Transfer timed out, NAK";
+            case LIBUSB_ERROR_BUSY :
+                msg = "LIBUSB_ERROR_BUSY: Resource busy.";
                 break;
-#endif
-            case -ENODEV:
-                msg = "-ENODEV: Device was removed";
+            case LIBUSB_ERROR_TIMEOUT :
+                msg = "LIBUSB_ERROR_TIMEOUT: Operation timed out.";
                 break;
-            case -EIO:
-                msg = "-EIO: USB I/O error";
+            case LIBUSB_ERROR_OVERFLOW :
+                msg = "LIBUSB_ERROR_OVERFLOW: Overflow.";
                 break;
-#ifdef EREMOTEIO
-            case -EREMOTEIO:
-                msg = "-EREMOTEIO: Short packet detected";
+            case LIBUSB_ERROR_PIPE :
+                msg = "LIBUSB_ERROR_PIPE: Pipe error.";
                 break;
-#endif
-            case -EXDEV:
-                msg = "-EXDEV: ISO transfer only partially completed look at "
-                      "individual frame status for details";
+            case LIBUSB_ERROR_INTERRUPTED :
+                msg = "LIBUSB_ERROR_INTERRUPTED: System call interrupted (perhaps due to signal)";
                 break;
-            case -EINVAL:
-                msg = "-EINVAL: ISO madness, if this happens: Log off and go home";
+            case LIBUSB_ERROR_NO_MEM :
+                msg = "LIBUSB_ERROR_NO_MEM: Insufficient memory.";
                 break;
+            case LIBUSB_ERROR_NOT_SUPPORTED :
+                msg = "LIBUSB_ERROR_NOT_SUPPORTED: Operation not supported or unimplemented on this platform.";
+                break;
+            case LIBUSB_ERROR_OTHER :
+                msg = "LIBUSB_ERROR_OTHER: Other error.";
+                break;
+
             default:
                 msg = "Unknown error";
                 break;
         }
-
-        DEBUG( "%s 0x%08x (%d)\n", msg, result, result );
+        DEBUG( "%s ERR: %s 0x%08x (%d)\n", function, msg, result, result );
     }
 }
 
