@@ -21,14 +21,7 @@
 #ifndef __DFU_H__
 #define __DFU_H__
 
-#if HAVE_CONFIG_H
-# include <config.h>
-#endif
-#ifdef HAVE_LIBUSB_1_0
-#include <libusb.h>
-#else
-#include <usb.h>
-#endif
+#include <libusb-1.0/libusb.h>
 #include <stdint.h>
 #include <stddef.h>
 
@@ -83,10 +76,20 @@ typedef struct {
     uint8_t iString;
 } dfu_status_t;
 
+void dfu_set_transaction_num( uint16_t newNum );
+/* set / reset the wValue parameter to a given value. this number is
+ * significant for stm32 device commands (see dfu-device.h)
+ */
+
+uint16_t dfu_get_transaction_num( void );
+/* get the current transaction number (can be used to calculate address
+ * offset for stm32 devices --- see dfu-device.h)
+ */
+
 int32_t dfu_detach( dfu_device_t *device, const int32_t timeout );
 /*  DFU_DETACH Request (DFU Spec 1.0, Section 5.1)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *  timeout   - the timeout in ms the USB device should wait for a pending
  *              USB reset before giving up and terminating the operation
  *
@@ -96,7 +99,7 @@ int32_t dfu_detach( dfu_device_t *device, const int32_t timeout );
 int32_t dfu_download( dfu_device_t *device, const size_t length, uint8_t* data );
 /*  DFU_DNLOAD Request (DFU Spec 1.0, Section 6.1.1)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *  length    - the total number of bytes to transfer to the USB
  *              device - must be less than wTransferSize
  *  data      - the data to transfer
@@ -107,7 +110,7 @@ int32_t dfu_download( dfu_device_t *device, const size_t length, uint8_t* data )
 int32_t dfu_upload( dfu_device_t *device, const size_t length, uint8_t* data );
 /*  DFU_UPLOAD Request (DFU Spec 1.0, Section 6.2)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *  length    - the maximum number of bytes to receive from the USB
  *              device - must be less than wTransferSize
  *  data      - the buffer to put the received data in
@@ -118,7 +121,7 @@ int32_t dfu_upload( dfu_device_t *device, const size_t length, uint8_t* data );
 int32_t dfu_get_status( dfu_device_t *device, dfu_status_t *status );
 /*  DFU_GETSTATUS Request (DFU Spec 1.0, Section 6.1.2)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *  status    - the data structure to be populated with the results
  *
  *  return the 0 if successful or < 0 on an error
@@ -127,7 +130,7 @@ int32_t dfu_get_status( dfu_device_t *device, dfu_status_t *status );
 int32_t dfu_clear_status( dfu_device_t *device );
 /*  DFU_CLRSTATUS Request (DFU Spec 1.0, Section 6.1.3)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *
  *  return 0 or < 0 on an error
  */
@@ -135,7 +138,7 @@ int32_t dfu_clear_status( dfu_device_t *device );
 int32_t dfu_get_state( dfu_device_t *device );
 /*  DFU_GETSTATE Request (DFU Spec 1.0, Section 6.1.5)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *
  *  returns the state or < 0 on error
  */
@@ -143,17 +146,13 @@ int32_t dfu_get_state( dfu_device_t *device );
 int32_t dfu_abort( dfu_device_t *device );
 /*  DFU_ABORT Request (DFU Spec 1.0, Section 6.1.4)
  *
- *  device    - the dfu device to commmunicate with
+ *  device    - the dfu device to communicate with
  *
  *  returns 0 or < 0 on an error
  */
 
 
-#ifdef HAVE_LIBUSB_1_0
 struct libusb_device
-#else
-struct usb_device
-#endif
                      *dfu_device_init( const uint32_t vendor,
                                        const uint32_t product,
                                        const uint32_t bus,
@@ -166,7 +165,7 @@ struct usb_device
  *
  *  vendor  - the vender number of the device to look for
  *  product - the product number of the device to look for
- *  [out] device - the dfu device to commmunicate with
+ *  [out] device - the dfu device to communicate with
  *
  *  return a pointer to the usb_device if found, or NULL otherwise
  */
