@@ -18,16 +18,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#if HAVE_CONFIG_H
-# include <config.h>
-#endif
 #include <stdio.h>
 #include <string.h>
-#ifdef HAVE_LIBUSB_1_0
-#include <libusb.h>
-#else
-#include <usb.h>
-#endif
+#include <libusb-1.0/libusb.h>
 
 #include "config.h"
 #include "dfu-device.h"
@@ -38,9 +31,7 @@
 
 
 int debug;
-#ifdef HAVE_LIBUSB_1_0
-libusb_context *usbcontext;
-#endif
+libusb_context *usbContext;
 
 int main( int argc, char **argv )
 {
@@ -49,11 +40,7 @@ int main( int argc, char **argv )
     int status;
     dfu_device_t dfu_device;
     struct programmer_arguments args;
-#ifdef HAVE_LIBUSB_1_0
     struct libusb_device *device = NULL;
-#else
-    struct usb_device *device = NULL;
-#endif
 
     memset( &args, 0, sizeof(args) );
     memset( &dfu_device, 0, sizeof(dfu_device) );
@@ -67,33 +54,31 @@ int main( int argc, char **argv )
         return SUCCESS;
     }
 
-#ifdef HAVE_LIBUSB_1_0
-    if (libusb_init(&usbcontext)) {
+    if (libusb_init(&usbContext)) {
         fprintf( stderr, "%s: can't init libusb.\n", progname );
         return DEVICE_ACCESS_ERROR;
     }
-#else
-    usb_init();
-#endif
 
     if( debug >= 200 ) {
-#ifdef HAVE_LIBUSB_1_0
-        libusb_set_debug(usbcontext, debug );
-#else
-        usb_set_debug( debug );
-#endif
+    #if LIBUSB_API_VERSION >= 0x01000106
+        libusb_set_option(usbContext, LIBUSB_OPTION_LOG_LEVEL, debug);
+    #else
+        libusb_set_debug(usbContext, debug );
+    #endif
     }
 
-    device = dfu_device_init( args.vendor_id, args.chip_id,
-                              args.bus_id, args.device_address,
-                              &dfu_device,
-                              args.initial_abort,
-                              args.honor_interfaceclass );
+    if( !(args.command == com_bin2hex || args.command == com_hex2bin) ) {
+        device = dfu_device_init( args.vendor_id, args.chip_id,
+                                  args.bus_id, args.device_address,
+                                  &dfu_device,
+                                  args.initial_abort,
+                                  args.honor_interfaceclass );
 
-    if( NULL == device ) {
-        fprintf( stderr, "%s: no device present.\n", progname );
-        retval = DEVICE_ACCESS_ERROR;
-        goto error;
+        if( NULL == device ) {
+            fprintf( stderr, "%s: no device present.\n", progname );
+            retval = DEVICE_ACCESS_ERROR;
+            goto error;
+        }
     }
 
     if( 0 != (retval = execute_command(&dfu_device, &args)) ) {
@@ -105,11 +90,7 @@ error:
     if( NULL != dfu_device.handle ) {
         int rv;
 
-#ifdef HAVE_LIBUSB_1_0
         rv = libusb_release_interface( dfu_device.handle, dfu_device.interface );
-#else
-        rv = usb_release_interface( dfu_device.handle, dfu_device.interface );
-#endif
         /* The RESET command sometimes causes the usb_release_interface command to fail.
            It is not obvious why this happens but it may be a glitch due to the hardware
            reset in the attached device. In any event, since reset causes a USB detach
@@ -124,19 +105,10 @@ error:
     }
 
     if( NULL != dfu_device.handle ) {
-#ifdef HAVE_LIBUSB_1_0
         libusb_close(dfu_device.handle);
-#else
-        if( 0 != usb_close(dfu_device.handle) ) {
-            fprintf( stderr, "%s: failed to close the handle.\n", progname );
-            retval = DEVICE_ACCESS_ERROR;
-        }
-#endif
     }
 
-#ifdef HAVE_LIBUSB_1_0
-    libusb_exit(usbcontext);
-#endif
+    libusb_exit(usbContext);
 
     return retval;
 }
